package org.especialistajee.cw.filmoteca.dao;

import org.especialistajee.cw.filmoteca.domain.ActorBean;
import org.especialistajee.cw.filmoteca.domain.ActoresBean;
import org.especialistajee.cw.filmoteca.domain.PeliculaBean;
import org.especialistajee.cw.filmoteca.domain.PeliculasBean;


/**
 * Interfaz comun de los objetos que dan acceso a los datos de las peliculas.
 * @author malozano
 *
 */
public interface IPeliculaDao {

	/**
	 * Añade una pelicula.
	 * @param p Pelicula que se añadira. Se producira un error si la pelicula
	 * ya existe, o si el parametro es <code>null</code>.
	 * @return Identificador asignado a la pelicula
	 * @throws DaoException Si ocurre un error al añadir la pelicula
	 */
	public abstract int addPelicula(PeliculaBean p) throws DaoException;

	/**
	 * Actualiza una pelicula.
	 * @param p Pelicula que se actualizara. Se producira un error si el parametro 
	 * es <code>null</code>, o si no existe ninguna pelicula con su identificador.
	 * @return Numero de registros afectados
	 * @throws DaoException Si ocurre un error al actualizar la pelicula
	 */
	public abstract int updatePelicula(PeliculaBean p) throws DaoException;
	
	/**
	 * Elimina una pelicula.
	 * @param p Pelicula a eliminar. Se producira un error si la pelicula no
	 * existe, o si el parametro es <code>null</code>.
	 * @return Numero de registros afectados
	 * @throws DaoException Si ocurre un error al eliminar la pelicula
	 */
	public abstract int delPelicula(int idPelicula) throws DaoException;

	/**
	 * Devuelve la lista de todas las peliculas.
	 * @return Lista de todas las peliculas.
	 */
	public abstract PeliculasBean getPeliculas() throws DaoException;

	/**
	 * Devuelve una pelicula dado su identificador.
	 * @return Datos de la pelicula seleccionada.
	 */
	public abstract PeliculaBean getPelicula(int id) throws DaoException;

	/**
	 * Realiza una búsqueda de las películas por titulo
	 * @return Lista de peliculas encontradas.
	 */
	public abstract PeliculasBean getPeliculas(String busqueda) throws DaoException;

	
	/**
	 * Devuelve la lista de actores de una pelicula dada
	 * @param idPelicula Identificador de la pelicula
	 * @return Lista de actores
	 * @throws DaoException Si ocurre un error en el acceso a la base de datos, o la
	 * pelicula no existe.
	 */
	public abstract ActoresBean getActoresPelicula(int idPelicula) throws DaoException;
	
	/**
	 * Añade un actor a la pelicula indicada
	 * @param idPelicula Identificador de la pelicula
	 * @param actor Datos del actor a añadir
	 * @throws DaoException Si ocurre un error en el acceso a la base de datos, o
	 * la pelicula no existe
	 */
	public abstract void addActor(int idPelicula, ActorBean actor) throws DaoException;
	
	/**
	 * Borra todos los actores de una pelicula
	 * @param idPelicula Identificador de la pelicula
	 * @return Numero de registros afectados
	 * @throws DaoException Si ocurre un error en el acceso a la base de datos, o
	 * la pelicula no existe
	 */
	public abstract int delActoresPelicula(int idPelicula) throws DaoException;
}