package org.especialistajee.cw.filmoteca.dao;

import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import org.especialistajee.cw.filmoteca.domain.ActorBean;
import org.especialistajee.cw.filmoteca.domain.ActoresBean;
import org.especialistajee.cw.filmoteca.domain.PeliculaBean;
import org.especialistajee.cw.filmoteca.domain.PeliculasBean;


/**
 * Implementacion del acceso a los datos de las peliculas almacenados en una base de
 * datos.
 * @author malozano
 *
 */
public class JdbcPeliculaDao implements IPeliculaDao {


	public int addPelicula(PeliculaBean p) throws DaoException {
		Connection con=null;
		
		if(p==null || p.getTitulo()==null || p.getTitulo().equals("")) {
			throw new DaoException("Ni la pelicula ni su titulo pueden ser nulos ni vacios");
		}

		try { 
			con = FuenteDatosJdbc.getInstance().getConnection();
			
			PreparedStatement stmt = con.prepareStatement("INSERT INTO peliculas(titulo, duracion, fechaEstreno, director) VALUES(?, ?, ?, ?)",Statement.RETURN_GENERATED_KEYS);
			stmt.setString(1, p.getTitulo());
			stmt.setInt(2, p.getDuracion());
			if(p.getFechaEstreno()!=null) {
				stmt.setDate(3, new Date(p.getFechaEstreno().getTime()));				
			} else {
				stmt.setDate(3, null);								
			}
			stmt.setString(4, p.getDirector());

			stmt.executeUpdate();
			
			// Obtiene la clave generada
			ResultSet rs = stmt.getGeneratedKeys();
			if(rs.next()) {
				int id = rs.getInt(1);

				// Inserta actores
				if(p.getActores()!=null) {
					PreparedStatement stmtActores = con.prepareStatement("INSERT INTO actores(fk_pelicula, nombreActor, nombrePersonaje) VALUES(?, ?, ?)");

					for(ActorBean actor: p.getActores()) {
						stmtActores.clearParameters();
						stmtActores.setInt(1, id);
						stmtActores.setString(2, actor.getNombreActor());
						stmtActores.setString(3, actor.getPersonaje());
						
						stmtActores.executeUpdate();
					}
				}
								
				return id;
			} else {
				throw new DaoException("No se encuentra el id generado");				
			}
			
		} catch (SQLException e) {
			throw new DaoException("Error al agregar pelicula", e);
		} finally {
			if(con!=null) {
				try {
					con.close();
				} catch (SQLException e) {
				}
			}
		}	
	}

	public int updatePelicula(PeliculaBean p) throws DaoException {
		Connection con=null;
		
		if(p==null || p.getTitulo()==null || p.getTitulo().equals("")) {
			throw new DaoException("Ni la pelicula ni su titulo pueden ser nulos ni vacios");
		}

		try { 
			con = FuenteDatosJdbc.getInstance().getConnection();
			
			PreparedStatement ps = con.prepareStatement("UPDATE peliculas SET titulo=?, duracion=?, fechaEstreno=?, director=? WHERE id=?");
			ps.setString(1, p.getTitulo());
			ps.setInt(2, p.getDuracion());
			if(p.getFechaEstreno()!=null) {
				ps.setDate(3, new Date(p.getFechaEstreno().getTime()));				
			} else {
				ps.setDate(3, null);								
			}
			ps.setString(4, p.getDirector());
			ps.setInt(5, p.getId());

			return ps.executeUpdate();
						
		} catch (SQLException e) {
			throw new DaoException("Error al agregar pelicula", e);
		} finally {
			if(con!=null) {
				try {
					con.close();
				} catch (SQLException e) {
				}
			}
		}	
	}
	
	public int delPelicula(int idPelicula) throws DaoException {
		Connection con=null;
		try { 
			con = FuenteDatosJdbc.getInstance().getConnection();
			
			PreparedStatement stmt = con.prepareStatement("DELETE FROM peliculas WHERE id=?");
			stmt.setInt(1, idPelicula);
			
			return stmt.executeUpdate();
			
		} catch (SQLException e) {
			throw new DaoException("Error al eliminar pelicula", e);
		} finally {
			if(con!=null) {
				try {
					con.close();
				} catch (SQLException e) {
				}
			}
		}
	}

	public PeliculasBean getPeliculas() throws DaoException {
		Connection con = null;
		try {
			con = FuenteDatosJdbc.getInstance().getConnection();
			
			Statement stmt = con.createStatement();
			
			ResultSet rs = stmt.executeQuery("SELECT * FROM peliculas");
			List<PeliculaBean> lista = new ArrayList<PeliculaBean>();
			while(rs.next()) {
				PeliculaBean pelicula = creaPelicula(rs);	
				lista.add(pelicula);
			}
			
			PeliculasBean peliculas = new PeliculasBean();
			peliculas.setPeliculas(lista);
			return peliculas;
		} catch (SQLException e) {
			throw new DaoException("Error al obtener lista de peliculas", e);
		} finally {
			if(con!=null) {
				try {
					con.close();
				} catch (SQLException e) {
				}
			}
		}
	}

	public PeliculasBean getPeliculas(String busqueda) throws DaoException {
		Connection con = null;
		try {
			con = FuenteDatosJdbc.getInstance().getConnection();
			
			PreparedStatement ps = con.prepareStatement("SELECT * FROM peliculas WHERE titulo like ?");
			ps.setString(1, "%" + busqueda + "%");
			
			ResultSet rs = ps.executeQuery();
			List<PeliculaBean> lista = new ArrayList<PeliculaBean>();
			while(rs.next()) {
				PeliculaBean pelicula = creaPelicula(rs);	
				lista.add(pelicula);
			}

			PeliculasBean peliculas = new PeliculasBean();
			peliculas.setPeliculas(lista);
			return peliculas;
		} catch (SQLException e) {
			throw new DaoException("Error al obtener lista de peliculas", e);
		} finally {
			if(con!=null) {
				try {
					con.close();
				} catch (SQLException e) {
				}
			}
		}
	}
	
	public PeliculaBean getPelicula(int id) throws DaoException {
		Connection con = null;
		try {
			con = FuenteDatosJdbc.getInstance().getConnection();
			
			PreparedStatement ps = con.prepareStatement("SELECT * FROM peliculas WHERE id=?");
			ps.setInt(1, id);
			
			ResultSet rs = ps.executeQuery();
			if(rs.next()) {
				PeliculaBean pelicula = creaPelicula(rs);	
				return pelicula;
			} else {
				return null;
			}
		} catch (SQLException e) {
			throw new DaoException("Error al obtener pelicula", e);
		} finally {
			if(con!=null) {
				try {
					con.close();
				} catch (SQLException e) {
				}
			}
		}
	}
	
	public ActoresBean getActoresPelicula(int idPelicula) throws DaoException {
		Connection con = null;
		try {
			con = FuenteDatosJdbc.getInstance().getConnection();
			
			PreparedStatement ps = con.prepareStatement("SELECT * FROM actores WHERE fk_pelicula=?");
			ps.setInt(1, idPelicula);
			
			ResultSet rs = ps.executeQuery();
			List<ActorBean> lista = new ArrayList<ActorBean>();
			while(rs.next()) {
				ActorBean actor = creaActor(rs);	
				lista.add(actor);
			}
			
			ActoresBean actores = new ActoresBean();
			actores.setActores(lista);
			return actores;
		} catch (SQLException e) {
			throw new DaoException("Error al obtener lista de actores", e);
		} finally {
			if(con!=null) {
				try {
					con.close();
				} catch (SQLException e) {
				}
			}
		}
	}
	
	public void addActor(int idPelicula, ActorBean actor) throws DaoException {
		Connection con=null;
		
		try { 
			con = FuenteDatosJdbc.getInstance().getConnection();
			
			// Inserta actores
			PreparedStatement stmtActores = con.prepareStatement("INSERT INTO actores(fk_pelicula, nombreActor, nombrePersonaje) VALUES(?, ?, ?)");

			stmtActores.clearParameters();
			stmtActores.setInt(1, idPelicula);
			stmtActores.setString(2, actor.getNombreActor());
			stmtActores.setString(3, actor.getPersonaje());
					
			stmtActores.executeUpdate();
			
		} catch (SQLException e) {
			throw new DaoException("Error al agregar actor", e);
		} finally {
			if(con!=null) {
				try {
					con.close();
				} catch (SQLException e) {
				}
			}
		}	
	}
	
	public int delActoresPelicula(int idPelicula) throws DaoException {
		Connection con = null;
		try {
			con = FuenteDatosJdbc.getInstance().getConnection();
			
			PreparedStatement ps = con.prepareStatement("DELETE FROM actores WHERE fk_pelicula=?");
			ps.setInt(1, idPelicula);
			
			return ps.executeUpdate();
		} catch (SQLException e) {
			throw new DaoException("Error al obtener lista de actores", e);
		} finally {
			if(con!=null) {
				try {
					con.close();
				} catch (SQLException e) {
				}
			}
		}
	}
	
	private PeliculaBean creaPelicula(ResultSet rs) throws SQLException {
		PeliculaBean pelicula = new PeliculaBean();
		
		pelicula.setId(rs.getInt("id"));
		pelicula.setTitulo(rs.getString("titulo"));
		pelicula.setDuracion(rs.getInt("duracion"));
		pelicula.setFechaEstreno(rs.getDate("fechaEstreno"));
		pelicula.setDirector(rs.getString("director"));
		
		return pelicula;
	}

	private ActorBean creaActor(ResultSet rs) throws SQLException {
		ActorBean actor = new ActorBean();
		
		actor.setNombreActor(rs.getString("nombreActor"));
		actor.setPersonaje(rs.getString("nombrePersonaje"));
		
		return actor;
	}
	
}
